@@ -1,14 +1,11 @@
 package CGI::Lazy::DB;
 
 use strict;
-use warnings;
 
 use DBI;
 use CGI::Lazy::Globals;
 use Carp;
 
-no warnings qw(uninitialized redefine);
-
 #---------------------------------------------------------------------------------------
 sub config {
 	my $self = shift;
