@@ -1,14 +1,11 @@
 package CGI::Lazy::Javascript;
 
 use strict;
-use warnings;
 
 use CGI::Lazy::Globals;
 use CGI::Lazy::Javascript::JSONParser;
 use JavaScript::Minifier qw(minify);
 
-no warnings qw(uninitialized redefine);
-
 #javascript for ajax requests
 our $AJAXJS = q[
 function ajaxSend(request, outgoing, returnHandler, returnTarget) {
@@ -162,22 +159,22 @@ datasetController.prototype.pushRow = function(caller) {
 	var oldRownum = oldRow.rowIndex;
 	var newRownum = oldRownum +1;
 
-	if (! document.getElementById("row" + newRownum)) {
+	if (! document.getElementById(this.widgetID + "Row" + newRownum)) {
 		var newRow = table.insertRow(table.rows.length);
-		newRow.id = 'row'+newRownum;
+		newRow.id = this.widgetID + 'Row' + newRownum;
 		var newRownum = newRow.rowIndex;
 
 		for (var i = 0; i< oldRow.cells.length; i++) {
 			var oldCell;
 			var oldWidget;
 			var newCell;
-			var position = newRow.cells.length;
-			newCell 		= newRow.insertCell(position);
+			var position 	= newRow.cells.length;
+			newCell 	= newRow.insertCell(position);
 
 			for (var j=0; j< oldRow.cells[i].childNodes.length; j++) {
-				oldCell = oldRow.cells[i];
-				oldWidget = oldCell.childNodes[j];
-				newCell.align 		= oldCell.align;
+				oldCell 	= oldRow.cells[i];
+				oldWidget 	= oldCell.childNodes[j];
+				newCell.align 	= oldCell.align;
 
 				var fieldName;
 
@@ -185,19 +182,29 @@ datasetController.prototype.pushRow = function(caller) {
 					oldWidget.name 		= oldWidget.name.replace(/(.+)-(.+)--(\\d+\$)/, "\$1-:INSERT:\$2--\$3");
 					fieldName 		= oldWidget.id.replace(/\\d+\$/, ''); 
 				}
-				var newWidget		= oldWidget.cloneNode(true);
+
+				var newWidget	= oldWidget.cloneNode(true);
 
 				try {
 					newWidget.name		= fieldName + newRownum;
 					newWidget.id 		= fieldName + newRownum;
-					newWidget.value		= '';
+
+					var re = /^checkbox/;
+					if (re.test(newWidget.type)) {
+						newWidget.value	= oldWidget.value;
+					} else {
+						newWidget.value	= '';
+					}
+
 				} catch (e) {
+
 				}
 
 				if (oldWidget.name && oldWidget.id) {
 					try {	
 						this.validator[newWidget.id] = this.validator[oldWidget.id];
 					} catch (e) {
+
 					}
 				}
 
@@ -336,6 +343,56 @@ datasetController.prototype.compositeSearch = function() {
 
 };
 
+END
+
+our $CONTROLLERJS = <<END;
+
+function controllerController(ID, containerID, selectObject) {
+	this.widgetID = ID;
+	this.containerID = containerID;
+	this.selectObject = selectObject;
+}
+
+controllerController.prototype.constructor = controllerController;
+
+controllerController.prototype.select = function() {
+	var outgoing = {};
+	var selectRequest;
+
+	for (i in this.selectObject) {
+		var re = /^select/;
+		if (re.test(document.getElementById(this.selectObject[i].name).type)) {
+			if (this.selectObject[i].required && !document.getElementById(this.selectObject[i].name).options[document.getElementById(this.selectObject[i].name).selectedIndex].value) {
+				return;
+			}
+			outgoing[this.selectObject[i].name] = document.getElementById(this.selectObject[i].name).options[document.getElementById(this.selectObject[i].name).selectedIndex].value;
+			if (document.getElementById(this.selectObject[i].name).options[document.getElementById(this.selectObject[i].name).selectedIndex].value) {
+			}
+		} else {
+			if (this.selectObject[i].required && !document.getElementById(this.selectObject[i].name).value) {
+				return;
+			}
+			outgoing[this.selectObject[i].name] = document.getElementById(this.selectObject[i].name).value;
+		}
+	}
+
+	ajaxSend(selectRequest, outgoing, this.selectResults, this.containerID);
+}
+
+controllerController.prototype.selectResults = function (text, target) {
+	var incoming = JSON.parse(text);
+	for (widgetname in incoming.validator) {
+		var controller = eval(widgetname + 'Controller');
+		delete controller.validator;
+		controller.validator = incoming.validator[widgetname];
+	}
+
+	var html = incoming.html;
+
+	document.getElementById(target).innerHTML = html;
+}
+
+
 END
 
 #javascript for domloader
@@ -347,6 +404,7 @@ our %component = (
 		'CGI::Lazy::Widget::Dataset'		=> $DatasetJS,
 		'CGI::Lazy::Widget::DomLoader'		=> $DOMLOADJS,
 		'CGI::Lazy::Widget::Composite'		=> $COMPJS,
+		'CGI::Lazy::Widget::Controller'		=> $CONTROLLERJS,
 );
 
 #-------------------------------------------------------------------------------------------------
