@@ -1,13 +1,10 @@
 package CGI::Lazy::Javascript::JSONParser;
 
 use strict;
-use warnings;
 
 use Exporter;
 use base qw(Exporter);
 
-no warnings qw(uninitialized redefine);
-
 our @EXPORT = qw($JSONPARSER);
 
 our $JSONPARSER = <<END;
