@@ -1,15 +1,13 @@
 package CGI::Lazy::Widget;
 
 use strict;
-use warnings;
 
 use JSON;
 use CGI::Lazy::Globals;
 use CGI::Lazy::Widget::Dataset;
 use CGI::Lazy::Widget::DomLoader;
 use CGI::Lazy::Widget::Composite;
-
-no warnings qw(uninitialized redefine);
+use CGI::Lazy::Widget::Controller;
 
 #----------------------------------------------------------------------------------------
 sub ajaxBlank {
@@ -68,6 +66,14 @@ sub config {
 	return $self->q->config;
 }
 
+#----------------------------------------------------------------------------------------
+sub controller {
+	my $self = shift;
+	my $vars = shift;
+
+	return CGI::Lazy::Widget::Controller->new($self->q, $vars);
+}
+
 #----------------------------------------------------------------------------------------
 sub dataset {
 	my $self = shift;
@@ -355,9 +361,10 @@ sub preloadLookup {
                         my $output = $lookups->{$queryname}->{output};
 
                         my $orderby = $lookups->{$queryname}->{orderby};
-                        my $orderbystring = join ',', @$orderby;
-                        $query .= " order by " if $orderby;
-                        $query .=$orderbystring;
+
+			if ($orderby) {
+				$query .= " order by ".  join ',', @$orderby;
+			}
 
                         my $results;
 
